unit unitChannel;

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs,
  ExtCtrls, StdCtrls, ComCtrls, cbDSmixer, Gauges;

const
  GRAPH_WIDTH = 110;

type
  TFormChannel = class(TForm)
    Panel2: TPanel;
    Label14: TLabel;
    edtFileName: TEdit;
    btnBrowse: TButton;
    pnlControl: TPanel;
    Label2: TLabel;
    Label3: TLabel;
    Label1: TLabel;
    Label4: TLabel;
    Label5: TLabel;
    Label6: TLabel;
    Label7: TLabel;
    Label8: TLabel;
    Label11: TLabel;
    Label12: TLabel;
    Label13: TLabel;
    buttonPlay: TButton;
    buttonStop: TButton;
    trackVolume: TTrackBar;
    editFrom: TEdit;
    editTo: TEdit;
    checkLoop: TCheckBox;
    buttonFade: TButton;
    buttonPause: TButton;
    trackPan: TTrackBar;
    Panel1: TPanel;
    paintboxLeft: TPaintBox;
    paintboxRight: TPaintBox;
    Label9: TLabel;
    Label10: TLabel;
    trackFrequency: TTrackBar;
    pbPosition: TProgressBar;
    Timer1: TTimer;
    OpenDialog1: TOpenDialog;
    procedure buttonPlayClick(Sender: TObject);
    procedure buttonStopClick(Sender: TObject);
    procedure buttonFadeClick(Sender: TObject);
    procedure checkLoopClick(Sender: TObject);
    procedure trackVolumeChange(Sender: TObject);
    procedure Timer1Timer(Sender: TObject);
    procedure buttonPauseClick(Sender: TObject);
    procedure editFromExit(Sender: TObject);
    procedure trackPanChange(Sender: TObject);
    procedure paintboxLeftPaint(Sender: TObject);
    procedure FormCreate(Sender: TObject);
    procedure paintboxRightPaint(Sender: TObject);
    procedure trackFrequencyChange(Sender: TObject);
    procedure pbPositionMouseDown(Sender: TObject; Button: TMouseButton;
      Shift: TShiftState; X, Y: Integer);
    procedure FormDestroy(Sender: TObject);
    procedure FormClose(Sender: TObject; var Action: TCloseAction);
    procedure editToExit(Sender: TObject);
    procedure btnBrowseClick(Sender: TObject);
  private
    FWB: TcbDSMixerChannel;
    FDSM: TcbDSMixer;
    GraphData: array [0..GRAPH_WIDTH, 0..1, 0..1] of byte;

    procedure SetWB( Value: TcbDSMixerChannel );
    procedure CalcGraph(AudioPtr: Pointer; SamplesCount, Channel,
      SamplesPerPixel, AddOn: Integer; var GraphPoint, SamplePoint: Integer;
      var PointUpper, PointLower: Byte);
  public
    constructor Create(Sender: TComponent; DSM: TcbDSMixer);
    procedure ClearGraphData;
    procedure OnEndPlay(Sender: TObject);

    property wb: TcbDSMixerChannel read FWB write SetWB;
  end;

var
  FormChannel: TFormChannel;

implementation

{$R *.DFM}

constructor TFormChannel.Create(Sender: TComponent; DSM: TcbDSMixer);
begin
  FDSM := DSM;

  inherited Create(Sender);
end;

procedure TFormChannel.OnEndPlay(Sender: TObject);
begin
    Timer1.Enabled := false;
    ClearGraphData;
    paintboxLeft.Invalidate;
    paintboxRight.Invalidate;
    pbPosition.Position := 0;
end;

procedure TFormChannel.buttonPlayClick(Sender: TObject);
begin
  FWB.Play;
  trackFrequency.Position := FWB.Frequency;
  timer1.Enabled := true;
end;


procedure TFormChannel.buttonStopClick(Sender: TObject);
begin
  timer1.Enabled := false;
  FWB.Stop;
end;

procedure TFormChannel.buttonFadeClick(Sender: TObject);
begin
  FWB.Fade(5000, -5000);
end;

procedure TFormChannel.checkLoopClick(Sender: TObject);
begin
  FWB.Loop := checkLoop.Checked;
end;

procedure TFormChannel.trackVolumeChange(Sender: TObject);
begin
  FWB.Volume := -10000-trackVolume.Position;
end;

procedure TFormChannel.CalcGraph(AudioPtr: Pointer; SamplesCount, Channel,
  SamplesPerPixel, AddOn: Integer; var GraphPoint, SamplePoint: Integer;
  var PointUpper, PointLower: Byte);
var
  V: Byte;
begin
  while GraphPoint < GRAPH_WIDTH do
  begin
    while SamplePoint <= SamplesPerPixel do
    begin
      if (SamplesCount = 0) then
        exit;

      v := ShortInt(AudioPtr^) + AddOn;
      if v > PointUpper then PointUpper := v;
      if v < PointLower then PointLower := v;
      Inc(SamplePoint);
      Inc(PByte(AudioPtr), FWB.FileBlockAlign);
      Dec(SamplesCount);
    end;

    GraphData[GraphPoint, Channel, 0] := PointUpper;
    GraphData[GraphPoint, Channel, 1] := PointLower;
    PointUpper := 0;
    PointLower := 255;
    Inc(GraphPoint);
    SamplePoint := 1;
  end;
end;

procedure TFormChannel.Timer1Timer(Sender: TObject);
var
  GraphPoint,
  SamplePoint: Integer;
  Size1,
  Size2,
  CntCh,
  NumSamps: DWord;
  PointUpper,
  PointLower,
  AddOn: Byte;
  Audio1Ptr,
  Audio2Ptr: Pointer;
  Channels,
  SamplesPerPixel: Integer;
begin
  if not(FWB.Status = sPlaying) then exit;
  NumSamps := (FWB.FileSamplesPerSec * Timer1.Interval) div 1000;
  if FWB.FileNumChannels > 1 then
    Channels := 2
  else
    Channels := 1;

  SamplesPerPixel := NumSamps div GRAPH_WIDTH;
  if SamplesPerPixel = 0 then
    SamplesPerPixel := 1;

  if (FWB.FileBitsPerSample = 8) then
    AddOn := 0
  else
    AddOn := 128;

  FWB.Lock(0, NumSamps*FWB.FileBlockAlign, Audio1Ptr, Size1, Audio2Ptr, Size2);
  try
    for CntCh := 0 to Channels-1 do
    begin
      PointUpper := 0;
      PointLower := 255;
      GraphPoint := 0;
      SamplePoint := 1;
      if (Size1 > 0) then
        CalcGraph(Pointer(PChar(Audio1Ptr) + FWB.FileBlockAlign - 1 -
          (FWB.FileBitsPerSample div 8) * CntCh),
          Size1 div FWB.FileBlockAlign,
          CntCh,
          SamplesPerPixel,
          AddOn,
          GraphPoint,
          SamplePoint,
          PointUpper,
          PointLower);
      if (Size2 > 0) then
        CalcGraph(Pointer(PChar(Audio2Ptr) + FWB.FileBlockAlign - 1 -
          (FWB.FileBitsPerSample div 8) * CntCh),
          Size2 div FWB.FileBlockAlign,
          CntCh,
          SamplesPerPixel,
          AddOn,
          GraphPoint,
          SamplePoint,
          PointLower,
          PointUpper);
    end;
  finally
    FWB.Unlock;
  end;
  paintboxLeft.Invalidate;
  paintboxRight.Invalidate;

  pbPosition.Position := (FWB.Position * 100) div FWB.FileDuration;
{  if Abs(Min-128.0) > Abs(Max-128.0) then
    pbarVU.Position := Trunc(Sqrt(Abs(Min-128.0) / 128) * 100)
  else
    pbarVU.Position := Trunc(Sqrt(Abs(Max-128.0) / 128) * 100);}
end;

procedure TFormChannel.SetWB( Value: TcbDSMixerChannel );
begin
  FWB := Value;
  Caption := FWB.FileName;
end;

procedure TFormChannel.buttonPauseClick(Sender: TObject);
begin
  FWB.Pause;
  Timer1.Enabled := False;
end;

procedure TFormChannel.editFromExit(Sender: TObject);
begin
  try
    FWB.RangeStart := StrToInt(editFrom.Text);
  except
    on Exception do;
  end;
end;

procedure TFormChannel.trackPanChange(Sender: TObject);
begin
  FWB.Pan := trackPan.Position;
end;

procedure TFormChannel.FormCreate(Sender: TObject);
begin
  if not Assigned(FWB) then
  begin
    FWB := TcbDSMixerChannel.Create(FDSM);
    FWB.OnEndPlay := OnEndPlay;
  end;

  ClearGraphData;
end;

procedure TFormChannel.paintboxLeftPaint(Sender: TObject);
var
  Cnt: Integer;
begin
  for Cnt := 0 to GRAPH_WIDTH-1 do
  begin
    paintboxLeft.Canvas.MoveTo( Cnt, (255-GraphData[Cnt, 0, 0]) shr 2 );
    paintboxLeft.Canvas.LineTo( Cnt, (255-GraphData[Cnt, 0, 1]) shr 2 );
  end;
end;

procedure TFormChannel.paintboxRightPaint(Sender: TObject);
var
  Cnt: Integer;
begin
  for Cnt := 0 to GRAPH_WIDTH-1 do
  begin
    paintboxRight.Canvas.MoveTo( Cnt, (255-GraphData[Cnt, 1, 0]) shr 2 );
    paintboxRight.Canvas.LineTo( Cnt, (255-GraphData [Cnt, 1, 1]) shr 2 );
  end;
end;

procedure TFormChannel.trackFrequencyChange(Sender: TObject);
begin
  FWB.Frequency := trackFrequency.Position;
end;

procedure TFormChannel.pbPositionMouseDown(Sender: TObject;
  Button: TMouseButton; Shift: TShiftState; X, Y: Integer);
begin
  FWB.Pause;
  Timer1.Enabled := False;
  FWB.Position := (X * Integer(FWB.FileDuration)) div pbPosition.ClientWidth;
  pbPosition.Position := (FWB.Position * 100) div FWB.FileDuration;
end;

procedure TFormChannel.ClearGraphData;
var
  Cnt: Integer;
begin
  for Cnt := 0 to GRAPH_WIDTH-1 do
  begin
    GraphData[Cnt, 0, 0] := 128;
    GraphData[Cnt, 0, 1] := 128;
    GraphData[Cnt, 1, 0] := 128;
    GraphData[Cnt, 1, 1] := 128;
  end;
end;

procedure TFormChannel.FormDestroy(Sender: TObject);
begin
  timer1.Enabled := False;
  FWB.Free;
  FWB := nil;
end;

procedure TFormChannel.FormClose(Sender: TObject;
  var Action: TCloseAction);
begin
  Action := caFree;
end;

procedure TFormChannel.editToExit(Sender: TObject);
begin
  try
    FWB.RangeEnd := StrToInt(editTo.Text);
  except
    on Exception do;
  end;
end;

procedure TFormChannel.btnBrowseClick(Sender: TObject);
begin
  if OpenDialog1.Execute then
  begin
    edtFileName.Text := '';
    pnlControl.Enabled := False;
    FWB.FileName := '';

    FWB.FileName := OpenDialog1.FileName;
    edtFileName.Text := FWB.FileName;
    pnlControl.Enabled := True;
  end;
end;

end.
